<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

use Util\DateUtil;

class VinculoRepository {
    /** @var EntityManager */
    protected $entityManager;

    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->entityManager        = $entityManager;
        $this->connection           = $entityManager->getConnection();
    }

    public function getVinculoHistorico(int $nrorg, int $nrvinculom): Array {
        $sql = <<<SQL
            SELECT *
              FROM GPE_VINCULOM VM
              JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG AND VH.NRVINCULOM = VM.NRVINCULOM AND VH.DTMESCOMPETENC = (
                SELECT MAX(DTMESCOMPETENC)
                  FROM GPE_VINCULOH VHH
                 WHERE VHH.NRORG = VH.NRORG
                       AND VHH.NRVINCULOM = VH.NRVINCULOM
               )
             WHERE VM.NRORG = :P_NRORG
                   AND VM.NRVINCULOM = :P_NRVINCULOM
SQL;

        $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"       => $nrorg,
                    "P_NRVINCULOM"  => $nrvinculom
                )
            );

            $result = $query->fetchAll();

            return !empty($result) ? $result[0] : $result;
    }

    public function getVinculoMestreHistoricoByNrparcnegocio(int $nrorg, int $nrparcnegocio): Array {
        $sql = <<<SQL
            SELECT VM.*,
                   VH.*
              FROM GPE_VINCULOM VM
              JOIN GPE_PESSOA PM ON PM.NRORG = VM.NRORG AND PM.NRPESSOA = VM.NRPESSOA
              JOIN PARCNEGOCIO PN ON PN.NRORG = VM.NRORG AND PN.NRPARCNEGOCIO = PM.NRPARCNEGOCIO
              JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG AND VH.NRVINCULOM = VM.NRVINCULOM AND VH.DTMESCOMPETENC = (
                SELECT MAX(DTMESCOMPETENC)
                  FROM GPE_VINCULOH VHH
                 WHERE VHH.NRORG = VH.NRORG
                       AND VHH.NRVINCULOM = VH.NRVINCULOM
               )
             WHERE VM.NRORG = :P_NRORG
                   AND PN.NRPARCNEGOCIO = :P_NRPARCNEGOCIO
SQL;

        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"       => $nrorg,
                "P_NRPARCNEGOCIO"  => $nrparcnegocio
            )
        );

        $result = $query->fetchAll();

        return !empty($result) ? $result[0] : $result;
    }


    public function getOperadorVinculo($nrorg, $nrvinculom) {
        $sql = <<<SQL
            SELECT OP.CDOPERADOR
              FROM GPE_VINCULOM VM
              JOIN GPE_PESSOA PE ON PE.NRORG = VM.NRORG
                                AND PE.NRPESSOA = VM.NRPESSOA
              JOIN OPERADOR OP ON OP.NRORG = PE.NRORG
                              AND OP.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
             WHERE VM.NRORG = :P_NRORG
                AND VM.NRVINCULOM = :P_NRVINCULOM
SQL;

        $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"       => $nrorg,
                    "P_NRVINCULOM"  => $nrvinculom
                )
            );

            $result = $query->fetch();

            return $result;
    }

    public function getVinculoMestreHistorico(string $nrorg, string $nrvinculom) {
        $sql = <<<SQL
            SELECT VM.*,
                   VH.*
              FROM GPE_VINCULOM VM
              JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG AND VH.NRVINCULOM = VM.NRVINCULOM AND VH.DTMESCOMPETENC = (
                SELECT MAX(DTMESCOMPETENC)
                  FROM GPE_VINCULOH VHH
                 WHERE VHH.NRORG = VH.NRORG
                       AND VHH.NRVINCULOM = VH.NRVINCULOM
               )
             WHERE VM.NRORG = :P_NRORG
                   AND VM.NRVINCULOM = :P_NRVINCULOM
SQL;

        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"       => $nrorg,
                "P_NRVINCULOM"  => $nrvinculom
            )
        );

        $result = $query->fetchAll();

        return !empty($result) ? $result[0] : $result;
    }

    /**
     *  Rotina responsável por buscar a escala ativa de horários do funcionário. A escala é buscada a partir das alterações de escala registradas para o vínculo,
     * Disponíveis na tabela 'GPE_ALTEESCALA'
     *
     *
     *  @param  int     $nrOrg
     *  @param  int     $nrVinculo
     *  @param  string  $dtMesCompetenc = null
     *
     *  @return array|false
     */
    public function getHorarioBase(
        int     $nrOrg,
        int     $nrVinculo,
        string  $date
    ) {
        $sql = <<<SQL
            SELECT
                EH.PRIENTRADA,
                EH.PRISAIDA,
                EH.SEGENTRADA,
                EH.SEGSAIDA,
                PO.HRTEMPOTOLERANCIAANT,
                PO.HRTEMPOTOLERANCIAPOS
            FROM
                GPE_VINCULOM VM

                JOIN GPE_ALTEESCALA   AE ON AE.NRORG = VM.NRORG AND AE.NRVINCULOM = VM.NRVINCULOM AND AE.DTINIESCALA = (
                    SELECT MAX(DTINIESCALA) FROM GPE_ALTEESCALA
                    WHERE GPE_ALTEESCALA.DTINIESCALA <= TO_DATE(:P_DATE, 'DD/MM/YYYY')
                      AND GPE_ALTEESCALA.NRVINCULOM = AE.NRVINCULOM
                      AND GPE_ALTEESCALA.NRORG = AE.NRORG
                )

                JOIN
                GPE_ESCALATRABM EM ON
                    EM.NRORG = VM.NRORG AND
                    EM.NRESCALATRABM = AE.NRESCALATRABM
                JOIN
                GPE_ESCALATRABH EH ON
                    EM.NRESCALATRABM = EH.NRESCALATRABM AND
                    EM.NRORG = EH.NRORG AND
                    EH.IDATIVO = 'S' AND
                    EH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            GPE_ESCALATRABH
                        WHERE
                            NRORG = EH.NRORG AND
                            NRESCALATRABM = EH.NRESCALATRABM AND
                            DTMESCOMPETENC <= SYSDATE AND
                            (
                                EM.DTFIMVIGENCIA >= SYSDATE OR
                                EM.DTFIMVIGENCIA IS NULL
                            )
                    )
                JOIN
                GPE_PARAMORG PO ON
                    VM.NRORG = PO.NRORG
                WHERE
                    VM.NRVINCULOM = :P_NRVINCULO AND
                    VM.NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);

        $std->execute(
            array(
                'P_NRORG'           => $nrOrg,
                'P_NRVINCULO'       => $nrVinculo,
                'P_DATE'            => $date,
            )
        );

        $horarioBase = $std->fetch();

        return $horarioBase;
    }

    public function getNrParcNegocioVinculo(
        array $params
    ) {
        $sql = <<<SQL
            SELECT
                P.NRPARCNEGOCIO
            FROM
                GPE_VINCULOM VM
                JOIN
                GPE_PESSOA P ON
                    P.NRPESSOA = VM.NRPESSOA AND
                    P.NRORG = VM.NRORG
            WHERE
                VM.NRVINCULOM = :P_NRVINCULO AND
                VM.NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetch();
        $nrParcNegocio = $result['NRPARCNEGOCIO'];

        return $nrParcNegocio;
    }

    public function isDiaFeriadoVinculo(
        int     $nrOrg,
        int     $nrVinculo,
        string  $dtmescompetenc,
        string  $dia = null
    ){
        $sql = <<<SQL
            SELECT
                *
            FROM
                OCORCALENDARIO
            WHERE
                NRCALENDARIO = (
                    SELECT
                        NRCALENDARIO
                    FROM
                        ESTRUTURAH
                    WHERE
                        NRESTRUTURAM = (
                            SELECT
                                NRESTRUTLEGAL
                            FROM
                                GPE_VINCULOH
                            WHERE
                                NRVINCULOM = :P_NRVINCULOM
                                AND NRORG = :P_NRORG
                                AND DTMESCOMPETENC IN (
                                    SELECT
                                        MAX(DTMESCOMPETENC)
                                    FROM
                                        GPE_VINCULOH
                                    WHERE
                                        NRVINCULOM = :P_NRVINCULOM
                                        AND NRORG = :P_NRORG
                                        AND DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                                )
                        )
                        AND DTMESCOMPETENC IN (
                            SELECT
                                MAX(DTMESCOMPETENC)
                            FROM
                                ESTRUTURAH H1
                            WHERE
                                H1.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM
                                AND H1.NRORG = ESTRUTURAH.NRORG
                                AND H1.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                        )
                        AND NRORG = :P_NRORG
                )
                AND NRORG = :P_NRORG
                AND ( TO_CHAR(DTCALENDARIO, 'DD/MM/YYYY') = :P_DTCALENDARIO
                      OR :P_DTCALENDARIO IS NULL )
SQL;

        $std = $this->connection->prepare($sql);

        $std->execute(
            array(
                "P_NRORG"           => $nrOrg,
                "P_NRVINCULOM"      => $nrVinculo,
                "P_DTMESCOMPETENC"  => $dtmescompetenc,
                "P_DTCALENDARIO"    => $dia,
            )
        );

        $result = $std->fetchAll();

        return $result;
    }

    public function setVinculo(
        array $params
    ) {
        $sql = <<<SQL
            INSERT INTO GPE_VINCULOM (
                NRVINCULOM,             NRPESSOA,           NRORG,
                DTADMISSAOVINC,         DTRESCISAOVINC,     CDMATRICULA,
                DTINCLUSAO,             DTULTATU,           NRORGINCLUSAO,
                CDOPERINCLUSAO,         NRORGULTATU,        CDOPERULTATU,
                NRTIPOVINCULOM,         NRVINCULOEMPREG,    NRTPADMISSAO,
                NRTPDEMISSAO,           DTOPCAOFGTS,        DTAPOSENTAFGTS,
                DTPRIMADMISSAO,         NMVINCULOM,         NRMOTIVORESC,
                DSOBSRESCISAO,          DTAVISOPREVIO,      NRDIASCONTRATOEXP,
                NRDIASCONTRATOEXPPRO,   DTEXAMEMEDICO,      DTAVISOTRCT,
                DSLIVROADMISSAO,        IDSEGDESEMPVINC,    NRDIASAPURVALES,
                NRTPAVIPRE,             DTENTREGAEPI,       DSOBSERVACAORESCISAO,
                IDVERIFICARESCISAO,     IDTPREINTEGRACAO,   NRPROCJUD,
                NRLEIANISTIA,           IDPAGTOJUIZO,       DTRETORNO,
                DTEFEITOFINANCEIRO,     OBSREINTEGRACAO,    DTCANCAVIPRE,
                NRMOTIVOCANCAP,         DSOBSCANCAVIPRE,    DTESOCIAL,
                DTVENCCONTREXPER,       NRCLAUSASSECURAT,   NRMOTIVOADMISSAO,
                NRVINCULOMSUBST
            )
            VALUES (
                :NRVINCULOM,            :NRPESSOA,          :NRORG,
                :DTADMISSAOVINC,        :DTRESCISAOVINC,    :CDMATRICULA,
                :DTINCLUSAO,            :DTULTATU,          :NRORGINCLUSAO,
                :CDOPERINCLUSAO,        :NRORGULTATU,       :CDOPERULTATU,
                :NRTIPOVINCULOM,        :NRVINCULOEMPREG,   :NRTPADMISSAO,
                :NRTPDEMISSAO,          :DTOPCAOFGTS,       :DTAPOSENTAFGTS,
                :DTPRIMADMISSAO,        :NMVINCULOM,        :NRMOTIVORESC,
                :DSOBSRESCISAO,         :DTAVISOPREVIO,     :NRDIASCONTRATOEXP,
                :NRDIASCONTRATOEXPPRO,  :DTEXAMEMEDICO,     :DTAVISOTRCT,
                :DSLIVROADMISSAO,       :IDSEGDESEMPVINC,   :NRDIASAPURVALES,
                :NRTPAVIPRE,            :DTENTREGAEPI,      :DSOBSERVACAORESCISAO,
                :IDVERIFICARESCISAO,    :IDTPREINTEGRACAO,  :NRPROCJUD,
                :NRLEIANISTIA,          :IDPAGTOJUIZO,      :DTRETORNO,
                :DTEFEITOFINANCEIRO,    :OBSREINTEGRACAO,   :DTCANCAVIPRE,
                :NRMOTIVOCANCAP,        :DSOBSCANCAVIPRE,   :DTESOCIAL,
                :DTVENCCONTREXPER,      :NRCLAUSASSECURAT,  :NRMOTIVOADMISSAO,
                :NRVINCULOMSUBST
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function setVinculoH(
        array $params
    ) {
        $sql = <<< SQL
            INSERT INTO GPE_VINCULOH (
                NRVINCULOH,             NRVINCULOM,         NRSITUFUNCM,
                NRCARGO,                NRESCALATRABM,      NRORG,
                DTMESCOMPETENC,         IDCONTRIBUISIND,    DTINCLUSAO,
                DTULTATU,               NRORGINCLUSAO,      CDOPERINCLUSAO,
                NRORGULTATU,            CDOPERULTATU,       NRDEPENDIR,
                NRDEPENDSFAM,           DTBASEFERIAS,       IDREMUNERACAO,
                IDTPPAGAMENTO,          NRESTRUTLEGAL,      NRESTRUTGEREN,
                NRTPMODALIDSAL,         IDMULTVINC,         NRFUNCAO,
                CDCONTRIBINDIVIDUAL,    NRTPMOVTRANSFM,     DTFIMCONTRDETERMIN,
                NRESTRUTSIND,           CONFIDENCIAL,       NRDIASAPURVALES,
                NRTURNO,                NRTPREGIMEPREV,     IDASSOCSINDICATO,
                CDMATRICULAESOCIAL,     NRFAIXA,            IDPAGPROPFER13,
                NRAGENTENOCIVO
            )
            VALUES (
                :NRVINCULOH,            :NRVINCULOM,        :NRSITUFUNCM,
                :NRCARGO,               :NRESCALATRABM,     :NRORG,
                :DTMESCOMPETENC,        :IDCONTRIBUISIND,   :DTINCLUSAO,
                :DTULTATU,              :NRORGINCLUSAO,     :CDOPERINCLUSAO,
                :NRORGULTATU,           :CDOPERULTATU,      :NRDEPENDIR,
                :NRDEPENDSFAM,          :DTBASEFERIAS,      :IDREMUNERACAO,
                :IDTPPAGAMENTO,         :NRESTRUTLEGAL,     :NRESTRUTGEREN,
                :NRTPMODALIDSAL,        :IDMULTVINC,        :NRFUNCAO,
                :CDCONTRIBINDIVIDUAL,   :NRTPMOVTRANSFM,    :DTFIMCONTRDETERMIN,
                :NRESTRUTSIND,          :CONFIDENCIAL,      :NRDIASAPURVALES,
                :NRTURNO,               :NRTPREGIMEPREV,    :IDASSOCSINDICATO,
                :CDMATRICULAESOCIAL,    :NRFAIXA,           :IDPAGPROPFER13,
                :NRAGENTENOCIVO
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function getVinculoTodos(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                VM.NRVINCULOM,
                VM.DTADMISSAOVINC,
                VM.DTRESCISAOVINC,
                VM.NMVINCULOM,
                VM.NRPESSOA,
                VM.NRORG,
                VM.NRTIPOVINCULOM,
                DECODE(VM.DTRESCISAOVINC, NULL, 0, 1) AS FILTRO
            FROM
                GPE_VINCULOM VM
            WHERE
                VM.NRORG = :P_NRORG
                AND EXISTS
                (
                    SELECT
                        1
                    FROM
                        GPE_VINCULOH
                    WHERE
                        GPE_VINCULOH.NRORG = VM.NRORG
                        AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                )
                AND VM.DTADMISSAOVINC <= LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
                AND (
                    VM.DTRESCISAOVINC >= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') OR
                    VM.DTRESCISAOVINC IS NULL
                )
            ORDER BY
                FILTRO ASC,
                NMVINCULOM ASC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $vinculos = $std->fetchAll();

        return $vinculos;
    }

    public function getVinculoTodosResp(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                VM.NRVINCULOM,
                VM.DTADMISSAOVINC,
                VM.DTRESCISAOVINC,
                VM.NMVINCULOM,
                VM.NRPESSOA,
                VM.NRORG,
                VM.NRTIPOVINCULOM,
                DECODE(VM.DTRESCISAOVINC, NULL, 0, 1) AS FILTRO
            FROM
                GPE_VINCULOM VM
                JOIN
                GPE_VINCULOH VH ON
                    VH.NRORG = VM.NRORG AND
                    VH.NRVINCULOM = VM.NRVINCULOM AND
                    VH.DTMESCOMPETENC = (
                        SELECT
                            MAX(VH1.DTMESCOMPETENC)
                        FROM
                            GPE_VINCULOH VH1
                        WHERE
                            VH1.NRORG = VH.NRORG AND
                            VH1.NRVINCULOM = VH.NRVINCULOM
                    )
            WHERE
                VM.NRORG = :P_NRORG
                AND VM.DTADMISSAOVINC <= LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
                AND EXISTS
                (
                    SELECT
                        1
                    FROM
                        GPE_VINCULOH
                    WHERE
                        GPE_VINCULOH.NRORG = VM.NRORG
                        AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                )
                AND (
                    VM.DTRESCISAOVINC > TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') OR
                    VM.DTRESCISAOVINC IS NULL
                )
                AND (INSTR(:P_NRESTRUTURAS, ',' || VH.NRESTRUTGEREN || ',')) > 0
            ORDER BY
                FILTRO ASC,
                NMVINCULOM ASC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $vinculos = $std->fetchAll();

        return $vinculos;
    }
}